package org.rcisoft.business.system.service.impl;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.mainte.adaptive.dao.BusParamReferRepository;
import org.rcisoft.business.mainte.adaptive.dao.BusTemperatureRepository;
import org.rcisoft.business.mainte.adaptive.dao.TotalSensorRepository;
import org.rcisoft.business.mainte.adaptive.entity.BusParamRefer;
import org.rcisoft.business.mainte.adaptive.entity.BusTemperature;
import org.rcisoft.business.mainte.adaptive.entity.TotalSensor;
import org.rcisoft.business.mainte.adaptive.proenum.ProEnum;
import org.rcisoft.business.mainte.adaptive.service.AdaptiveService;
import org.rcisoft.business.mainte.adaptive.vo.Params;
import org.rcisoft.business.overview.dao.BusProjectRepository;
import org.rcisoft.business.overview.dao.EnergyCountMRepository;
import org.rcisoft.business.overview.entity.BusProject;
import org.rcisoft.business.overview.vo.EnergyCountMVo;
import org.rcisoft.business.system.service.SystemService;
import org.rcisoft.business.system.vo.ClimateStatistics;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Calendar;
import java.util.List;

/**
 * Created by JiChao on 2018/5/2.
 */
@Service
public class SystemServiceImpl implements SystemService {
    @Autowired
    private BusParamReferRepository busParamReferRepository;
    @Autowired
    private TotalSensorRepository totalSensorRepository;
    @Autowired
    private BusTemperatureRepository busTemperatureRepository;
    @Autowired
    private BusProjectRepository busProjectRepository;
    @Autowired
    private EnergyCountMRepository energyCountMRepository;
    @Autowired
    private AdaptiveService adaptiveServiceImpl;

    private String getCode(String proId) {
        BusProject b = new BusProject();
        b.setProId(proId);
        BusProject busProject = busProjectRepository.selectOne(b);
        return busProject.getCode();
    }

    @Override
    public ClimateStatistics statClimate(String proId) {
        Calendar cal = Calendar.getInstance();
        String time = cal.get(Calendar.YEAR) + "-" + (cal.get(Calendar.MONTH) + 1) + "-" + cal.get(Calendar.DAY_OF_MONTH);
        //当前时间
        int nowHour = cal.get(Calendar.HOUR_OF_DAY);
        Params params = new Params(proId, time, this.getCode(proId));
        //查询当天气温
        List<BusTemperature> busTemperaturesList = busTemperatureRepository.queryBusTemperatureList(params);
        List<Object> outsideList = Arrays.asList(new Object[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0});
        busTemperaturesList.forEach(busTemperature -> {
            cal.setTime(busTemperature.getTm());
            int hour = cal.get(Calendar.HOUR_OF_DAY);
            if (hour > nowHour) return;
            outsideList.set(hour, busTemperature.getTemperature());
        });
        //查询 供水温度，回水温度 code
        String[] code_array = new String[2];
        List<BusParamRefer> busParamRefer = busParamReferRepository.queryOtherParam(params);
        busParamRefer.forEach(b -> {
            if(StringUtils.equals(b.getOwnParam(), ProEnum.gswd.toString())) code_array[0] = b.getOtherParam();
            if(StringUtils.equals(b.getOwnParam(), ProEnum.hswd.toString())) code_array[1] = b.getOtherParam();
        });
        //查询供回水温度
        List<TotalSensor> totalSensorList = totalSensorRepository.queryTotalSensorList(params);
        List<Object> gswdList = Arrays.asList(new Object[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0});
        List<Object> hswdList = Arrays.asList(new Object[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0});
        totalSensorList.forEach(totalSensor -> {
            cal.setTime(totalSensor.getTm());
            int hour = cal.get(Calendar.HOUR_OF_DAY);
            if (hour > nowHour) return;
            //得到json对象
            JSONObject json = JSONObject.parseObject(totalSensor.getSensorJson());
            //供水温度
            BigDecimal gswd = (BigDecimal) json.get(code_array[0]);
            //回水温度
            BigDecimal hswd = (BigDecimal) json.get(code_array[1]);
            gswdList.set(hour, gswd);
            hswdList.set(hour, hswd);
        });
        return new ClimateStatistics(outsideList, gswdList, hswdList);
    }

    @Override
    public List<Object> energyConsume(String proId) {
        Calendar cal = Calendar.getInstance();
//        cal.set(2018, 3, 28);
        String time = cal.get(Calendar.YEAR) + "-" + (cal.get(Calendar.MONTH) + 1) + "-" + cal.get(Calendar.DAY_OF_MONTH);
        EnergyCountMVo e = new EnergyCountMVo();
        e.setProId(proId);
        e.setTime(time);
        List<EnergyCountMVo> energyList = energyCountMRepository.energyConsume(e);
        List<Object> energyResultList = Arrays.asList(new Object[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0});
        energyList.forEach(energyCountMVo -> {
            int hour = Integer.parseInt(energyCountMVo.getHour());
            BigDecimal energy = energyCountMVo.getEnergy();
            energyResultList.set(hour, energy);
        });
        return energyResultList;
    }

    @Override
    public List<Object> burdenHour(String proId) {
        Calendar cal = Calendar.getInstance();
//        cal.set(2018, 3, 28);
        String time = cal.get(Calendar.YEAR) + "-" + (cal.get(Calendar.MONTH) + 1) + "-" + cal.get(Calendar.DAY_OF_MONTH);
        Params params = new Params(proId, time, this.getCode(proId));
        return adaptiveServiceImpl.buildingList(params);
    }
}
