package org.rcisoft.business.system.user.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.system.user.entity.*;
import org.rcisoft.business.system.user.service.UserService;
import org.rcisoft.business.system.user.vo.UserDto;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.controller.PaginationController;
import org.rcisoft.core.model.GridModel;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

/**
 * @Author: GaoLiWei
 * @Date: Created in 10:422018/5/2
 */
@Controller
@RequestMapping("user")
public class UserController  extends PaginationController<SysUser> {

    @Autowired
    private UserService userService;

    @ApiOperation(value="添加系统用户", notes="返回是否成功")
    @ApiImplicitParams({@ApiImplicitParam(name = "userDto", value = "Id不填,2是管理员，3是业主，4是团队负责人", required = true, dataType = "UserDto",paramType = "body")
    })
    @PostMapping(value = "saveUser")
    @ResponseBody
    public Result saveUser(@RequestBody UserDto userDto){
        SysUser sysUser = userDto.getSysUser();
        SysAdmins sysAdmins = userDto.getSysAdmins();
        SysOwner sysOwner = userDto.getSysOwner();
        SysPrincipal sysPrincipal = userDto.getSysPrincipal();
        int flag = userService.saveUser(sysUser, sysAdmins, sysOwner, sysPrincipal);
        return Result.builder(new PersistModel(flag));
    }

    @ApiOperation(value = "通过用户ID删除用户")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "userId", required = true,paramType = "path", dataType = "varchar"),
    })
    @ResponseBody
    @DeleteMapping(value = "/deleteUser/{userId}")
    public Result deleteUser(@PathVariable("userId") String userId){

        String data =userService.deleteUser(userId);
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR,data);
    }


    @ApiOperation(value="修改系统用户", notes="返回是否成功")
    @ApiImplicitParams({@ApiImplicitParam(name = "userDto", value = "2是管理员，3是业主，4是团队负责人", required = true, dataType = "UserDto",paramType = "body")
    })
    @PutMapping(value = "updateUser")
    @ResponseBody
    public Result updateUser(@RequestBody UserDto userDto){
        SysUser sysUser = userDto.getSysUser();
        SysAdmins sysAdmins = userDto.getSysAdmins();
        SysOwner sysOwner = userDto.getSysOwner();
        SysPrincipal sysPrincipal = userDto.getSysPrincipal();
        String data = userService.updateUser(sysUser,sysAdmins,sysOwner,sysPrincipal);
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR,data);
    }

    @ApiOperation(value = "通过用户ID查找用户信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "userId", required = true,paramType = "path", dataType = "varchar")
    })
    @ResponseBody
    @GetMapping(value = "/getUser/{userId}")
    public Result getUser(@PathVariable("userId") String userId){
        Object data = userService.getUserByUserId(userId);
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR,data);
    }


    @ApiOperation(value = "通过用户ID查找用户可以看到的菜单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "userId", required = true,paramType = "path", dataType = "varchar")
    })
    @ResponseBody
    @GetMapping(value = "/listMenu")
    public Result listMenu(){
        String userId = UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID);
        Object data = userService.listMenuBuUserId(userId);
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR,data);
    }

    @ApiOperation(value = "修改角色可以看到的菜单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "relRoleMenu", value = "用户id,菜单id", required = true,paramType = "body", dataType = "RelRoleMenu")
    })
    @ResponseBody
    @PostMapping(value = "/updateRoleMenu")
    public Result updateRoleMenu(@RequestBody RelRoleMenu relRoleMenu){
        int flag = userService.updateRoleMenu(relRoleMenu);
        return Result.builder(new PersistModel(flag));
    }



    @ApiOperation(value = "查詢所有菜單")
    @ResponseBody
    @GetMapping(value = "/listMenuAll")
    public Result listMenuAll(){
        Object data = userService.listMenuAll();
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR,data);
    }

    @ApiOperation(value = "查詢所有角色")
    @ResponseBody
    @GetMapping(value = "/listRoleAll")
    public Result listRoleAll(){
        Object data = userService.listRoleAll();
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR,data);
    }

    @ApiOperation(value = "查詢所有业主")
    @ResponseBody
    @GetMapping(value = "/listOnwerAll")
    public Result listOnwerAll(){
        Object data = userService.listOwnerAll();
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR,data);
    }

    @ApiOperation(value = "查詢所有团队")
    @ResponseBody
    @GetMapping(value = "/listTeamAll")
    public Result listTeamAll(){
        Object data = userService.listTeamAll();
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR,data);
    }

    @ApiOperation(value = "查詢所有用户")
    @ResponseBody
    @GetMapping(value = "/listUserAllByPagination")
    public GridModel listUserAllByPagination(@RequestParam String userTp){
        userService.listUserAllByPagination(getPaginationUtility(),userTp);
        return getGridModelResponse();
    }





}
