package org.rcisoft.business.system.user.service.impl;

import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.device.assets.entity.BusDevice;
import org.rcisoft.business.evaluate.team.dao.BusTeamRepository;
import org.rcisoft.business.manage.dao.SysUserProjectRepository;
import org.rcisoft.business.system.user.dao.*;
import org.rcisoft.business.system.user.entity.*;
import org.rcisoft.business.system.user.service.UserService;
import org.rcisoft.common.constants.SysRoleConstant;
import org.rcisoft.common.constants.UserPassWordConstant;
import org.rcisoft.common.constants.UserTpConstant;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.JwtUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author: GaoLiWei
 * @Date: Created in 10:432018/5/2
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class UserServiceImpl implements UserService {
    @Autowired
    private UserRepository userRepository;
    @Autowired
    private AdminRepository adminRepository;
    @Autowired
    private OwnerRepository ownerRepository;
    @Autowired
    private PrincipalRepository principalRepository;
    @Autowired
    private RelRoleMenuRepository relRoleMenuRepository;
    @Autowired
    private RelUserRoleRepository relUserRoleRepository;
    @Autowired
    private AuthenticationManager authenticationManager;
    @Autowired
    private UserDetailsService userDetailsService;
    @Autowired
    private PasswordEncoder passwordEncoder;
    @Autowired
    private RoleRepository roleRepository;
    @Autowired
    private BusTeamRepository busTeamRepository;
    @Autowired
    private SysUserProjectRepository sysUserProjectRepository;


    @Override
    public String login(String username, String password) {
        UsernamePasswordAuthenticationToken upToken = new UsernamePasswordAuthenticationToken(username, password);
        //进入到 UserDetailsService(UserDetailServiceImpl)  loadUserByUsername 方法
        final Authentication authentication = authenticationManager.authenticate(upToken);
        UserDetails userDetails = (UserDetails)authentication.getPrincipal();
        final String token = JwtUtil.generateToken(userDetails);
        SecurityContextHolder.getContext().setAuthentication(authentication);
        return token;
    }

    @Override
    public int saveUser(SysUser sysUser, SysAdmins sysAdmins, SysOwner sysOwner, SysPrincipal sysPrincipal) {
        int flag = 0;

        String userNm = sysUser.getUserNm();
        List<SysUser> flagSysUsers = userRepository.listByUserName(userNm);
        if (flagSysUsers.size() <= 0){
            String userId = IdGen.uuid();
            sysUser.setUserPwd(passwordEncoder.encode(UserPassWordConstant.PASSWORD));
            sysUser.setUserId(userId);

            String userTp = sysUser.getUserTp();
            try {
                if (UserTpConstant.TP_ADMIN.equals(userTp)) {
                    userRepository.insertSelective(sysUser);

                    sysAdmins.setAdminId(IdGen.uuid());
                    sysAdmins.setUserId(userId);
                    adminRepository.insertSelective(sysAdmins);

                    RelUserRole relUserRole = new RelUserRole();
                    relUserRole.setUserId(userId);
                    relUserRole.setRoleId(SysRoleConstant.TP_ADMIN_ID);
                    relUserRoleRepository.insertSelective(relUserRole);

                    flag = 1;
                }
                if (UserTpConstant.TP_OWNER.equals(userTp)) {
                    userRepository.insertSelective(sysUser);

                    sysOwner.setOwnId(IdGen.uuid());
                    sysOwner.setUserId(userId);
                    ownerRepository.insertSelective(sysOwner);

                    RelUserRole relUserRole = new RelUserRole();
                    relUserRole.setUserId(userId);
                    relUserRole.setRoleId(SysRoleConstant.TP_OWNER_ID);
                    relUserRoleRepository.insertSelective(relUserRole);

                    flag = 1;
                }
                if (UserTpConstant.TP_PRINCIPAL.equals(userTp)) {
                    userRepository.insertSelective(sysUser);

                    sysPrincipal.setPriId(IdGen.uuid());
                    sysPrincipal.setUserId(userId);
                    principalRepository.insertSelective(sysPrincipal);

                    RelUserRole relUserRole = new RelUserRole();
                    relUserRole.setUserId(userId);
                    relUserRole.setRoleId(SysRoleConstant.TP_PRINCIPAL_ID);
                    relUserRoleRepository.insertSelective(relUserRole);

                    flag = 1;
                }
                return flag;
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return flag;
    }

    @Override
    public String updateUser(SysUser sysUser, SysAdmins sysAdmins, SysOwner sysOwner, SysPrincipal sysPrincipal) {
        String result = "修改失败";
        String userId = sysUser.getUserId();
        String userTp = sysUser.getUserTp();
        try {
            if (UserTpConstant.TP_ADMIN.equals(userTp)) {
                SysAdmins sysAdminsByUserId = adminRepository.getSysAdminsByUserId(userId);
                if (null != sysAdminsByUserId){
                    sysAdmins.setAdminId(sysAdminsByUserId.getAdminId());
                    adminRepository.updateByPrimaryKeySelective(sysAdmins);
                    result = "修改管理员信息成功";
                }
            }
            if (UserTpConstant.TP_OWNER.equals(userTp)) {
                SysOwner sysOwnerByUserId = ownerRepository.getSysOwnerByUserId(userId);
                if (null != sysOwnerByUserId) {
                    sysOwner.setOwnId(sysOwnerByUserId.getOwnId());
                    ownerRepository.updateByPrimaryKeySelective(sysOwner);
                    result = "修改业主信息成功";
                }
            }
            if (UserTpConstant.TP_PRINCIPAL.equals(userTp)) {
                SysPrincipal sysPrincipalByUserId = principalRepository.getSysPrincipalByUserId(userId);
                if (null != sysPrincipalByUserId) {
                    sysPrincipal.setPriId(sysPrincipalByUserId.getPriId());
                    principalRepository.updateByPrimaryKeySelective(sysPrincipal);
                    result = "修改团队负责人信息成功";
                }
            }
            return result;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "修改失败";
    }

    @Override
    public String deleteUser(String userId) {
        String result = "删除失败";
        SysUser sysUser = new SysUser();
        sysUser.setUserId(userId);

        try {
            SysUser selectUser = userRepository.selectByPrimaryKey(sysUser);

            userRepository.deleteByPrimaryKey(sysUser);

            RelUserRole relUserRole = new RelUserRole();
            relUserRole.setUserId(userId);
            relUserRoleRepository.deleteByPrimaryKey(relUserRole);

            String userTp = selectUser.getUserTp();
            if (UserTpConstant.TP_ADMIN.equals(userTp)) {

                adminRepository.deleteSysAdminByUserId(userId);

                result = "删除成功";
            }
            if (UserTpConstant.TP_OWNER.equals(userTp)) {

                ownerRepository.deleteSysOwnerByUserId(userId);

                result = "删除成功";
            }
            if (UserTpConstant.TP_PRINCIPAL.equals(userTp)) {

                principalRepository.deleteSysPrincipalByUserId(userId);

                result = "删除成功";
            }
            return result;

        } catch (Exception e) {
            e.printStackTrace();
        }
        return "删除失败";
    }

    @Override
    public Object getUserByUserId(String userId) {

        SysUser sysUser = new SysUser();
        sysUser.setUserId(userId);
        try {
            SysUser selectUser = userRepository.selectByPrimaryKey(sysUser);

            String userTp = selectUser.getUserTp();
            if (UserTpConstant.TP_ADMIN.equals(userTp)) {
               return adminRepository.getSysAdminsByUserId(userId);
            }
            if (UserTpConstant.TP_OWNER.equals(userTp)) {
                return ownerRepository.getSysOwnerByUserId(userId);
            }
            if (UserTpConstant.TP_PRINCIPAL.equals(userTp)) {
                return principalRepository.getSysPrincipalByUserId(userId);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public List<Map<String, Object>> listMenuBuUserId(String userId) {
        List<Map<String, Object>> list =  relUserRoleRepository.listMenuBuUserId(userId);
        return handleMenu(list);
    }

    @Override
    public int updateRoleMenu(RelRoleMenu relRoleMenu) {
        int flag = relRoleMenuRepository.updateByPrimaryKeySelective(relRoleMenu);
        return flag;
    }

    @Override
    public List<Map<String, Object>> listMenuAll() {
        List<Map<String, Object>> list = relUserRoleRepository.listMenuAll();
        return handleMenu(list);
    }

    @Override
    public List<Map<String, Object>> listRoleAll() {
        //
        List<Map<String, Object>> list = roleRepository.listRoleAll();
        for(Map<String, Object> mp : list){
            String menuIds = mp.get("MENU_ID").toString();
            String[] menuArr = menuIds.split(",");
            List<String> menuList = new ArrayList<>();
            for(String s : menuArr){menuList.add(s);}
            mp.put("MENULIST",handleMenu(roleRepository.queryMenuByIds(menuList)));
        }
        return list;
    }

    @Override
    public List<Map<String, Object>> listTeamAll() {
        return busTeamRepository.listTeamAll();
    }

    @Override
    public List<Map<String, Object>> listOwnerAll() {
        return ownerRepository.listOwnerAll();
    }

    @Override
    public List<Map<String, Object>> listUserAllByPagination(PageUtil<SysUser> paginationUtility,String userTp) {
        String table  = "";
        if(userTp!=null && userTp.equals("1")){table = "sys_admins";}
        if(userTp!=null && userTp.equals("2")){table = "sys_owner";}
        if(userTp!=null && userTp.equals("3")){table = "sys_principal";}
        List<Map<String, Object>> list =  userRepository.listUserAll(table,userTp);
        for(Map<String, Object> mp : list){
            mp.put("REL_PROJECT",sysUserProjectRepository.querySysUserPro(mp.get("USER_ID").toString()));
        }
        return list;
    }

    public List<Map<String, Object>> handleMenu(List<Map<String, Object>> menuList){
        List<Map<String, Object>> result = new ArrayList<>();
        for(Map<String, Object> tmap : menuList){
            if(tmap.get("MENU_PID")==null){
                Map<String,Object> map = new HashMap<>();
                map.put("id",tmap.get("MENU_ID"));
                map.put("label",tmap.get("MENU_NM"));
                result.add(map);
            }
        }
        for(Map<String, Object> rmap : result){
            String pid = rmap.get("id").toString();
            List<Map<String, Object>> childList = new ArrayList<>();
            for(Map<String, Object> tmap : menuList){
                if(tmap.get("MENU_PID")!=null && tmap.get("MENU_PID").toString().equals(pid)){
                    Map<String, Object> m = new HashMap<>();
                    m.put("id",tmap.get("MENU_ID"));
                    m.put("label",tmap.get("MENU_NM"));
                    childList.add(m);
                }
            }
            rmap.put("children",childList);
        }
        return result;
    }
}
